#include <avr/io.h>
#include <avr/interrupt.h>  /* for sei() */
#include <util/delay.h>     /* for _delay_ms() */
#include <avr/pgmspace.h>   /* required by usbdrv.h */
#include "usbdrv.h"


typedef struct HidReport
{
/*
* +----\------+-----+-----+-----+-----+-----+-----+-----+-----+
* |Byte \ Bit | 7 | 6 | 5 | 4 | 3 | 2 | 1 | 0 |
* +------\----+-----+-----+-----+-----+-----+-----+-----+-----+
* | 0 | Light (L) |
* +-----------+-----+-----+-----+-----+-----+-----+-----+-----+
* | 1 | UNUSED | Btn | Light (H) |
* +-----------+-----+-----+-----+-----+-----+-----+-----+-----+
*
*/
uint8_t P1_R_top:1;
uint8_t P1_R_side:1;
uint8_t P1_R_mid:1;
uint8_t P1_L_top:1;
uint8_t P1_L_mid:1;
uint8_t P2_R_top:1;
uint8_t P2_R_side:1;
uint8_t P2_R_mid:1;
uint8_t P2_L_top:1;
uint8_t P2_L_mid:1;
uint8_t unused:6;
} HidReport_t;

static HidReport_t reportBuffer;
static uchar idleRate; // Unused

/////////////////////////////////////////////////////////////////////
// USB report descriptor, generated by HID Descriptor Tool from
//    http://www.usb.org/developers/hidpage/dt2_4.zip
/////////////////////////////////////////////////////////////////////
PROGMEM char usbHidReportDescriptor[USB_CFG_HID_REPORT_DESCRIPTOR_LENGTH] = {
    0x05, 0x01,                    // USAGE_PAGE (Generic Desktop)
    0x09, 0x04,                    // USAGE (Joystick)
    0xa1, 0x01,                    // COLLECTION (Application)
    0x05, 0x01,                    //     USAGE_PAGE (Generic Desktop)
    0x09, 0x01,                    //     USAGE (Pointer)
    0xa1, 0x00,                    //   COLLECTION (Physical)
    0x05, 0x09,                    //     USAGE_PAGE (Button)
    0x19, 0x01,                    //     USAGE_MINIMUM (Button 1)
    0x29, 0x0a,                    //     USAGE_MAXIMUM (Button 10)
    0x15, 0x00,                    //     LOGICAL_MINIMUM (0)
    0x25, 0x01,                    //     LOGICAL_MAXIMUM (1)
    0x95, 0x0a,                    //     REPORT_COUNT (10)
    0x75, 0x01,                    //     REPORT_SIZE (1)
    0x81, 0x02,                    //     INPUT (Data,Var,Abs)
    0x95, 0x06,                    //     REPORT_COUNT (6)
    0x75, 0x01,                    //     REPORT_SIZE (1)
    0x81, 0x03,                    //     INPUT (Cnts,Var,Abs)
    0xc0,                          //   END_COLLECTION
    0xc0                           // END_COLLECTION
};



///////////////////////////////////////////////////////////
uint16_t read_adc(uint8_t channel)
{
    uint16_t adc_val;

    ADMUX = 0x40;
    ADMUX |= channel;
    ADCSRA = 0xC6;
    while ((ADCSRA & (1<<ADSC)))
        ;
    adc_val = ADCL + ADCH*256;
    return adc_val;
}

///////////////////////////////////////////////////////////
void buildReport()
{
    reportBuffer.P1_R_top = ((PINC & (1<<PC0)) == 0);
    reportBuffer.P1_L_top = ((PINC & (1<<PC1)) == 0);
    reportBuffer.P1_R_side = ((PINC & (1<<PC2)) == 0);
    reportBuffer.P1_R_mid = ((PINC & (1<<PC4)) == 0);
    reportBuffer.P1_L_mid = ((PINC & (1<<PC5)) == 0);
    reportBuffer.P2_R_top = ((PINB & (1<<PB0)) == 0);
    reportBuffer.P2_L_top = ((PINB & (1<<PB1)) == 0);
    reportBuffer.P2_R_side = ((PINB & (1<<PB2)) == 0);
    reportBuffer.P2_R_mid = ((PINB & (1<<PB4)) == 0);
    reportBuffer.P2_L_mid = ((PINB & (1<<PB5)) == 0);

    //reportBuffer.light = read_adc(PC4);
}
///////////////////////////////////////////////////////////
usbMsgLen_t usbFunctionSetup(uint8_t data[8])
{
    usbRequest_t *rq = (void *)data;

    if(rq->bRequest == USBRQ_HID_GET_REPORT)
    {
        usbMsgPtr = (void *)&reportBuffer;
        return sizeof(reportBuffer);
    }
    else if (rq->bRequest == USBRQ_HID_GET_IDLE)
    {
        usbMsgPtr = &idleRate;
        return 1;
    }
    else if (rq->bRequest == USBRQ_HID_SET_IDLE)
    {
        idleRate = rq->wValue.bytes[1];
    }

    return 0;
}

///////////////////////////////////////////////////////////
int main(void)
{
    DDRC &= ~(1<<PC0);	//init PC0 to be input

    DDRC &= ~(1<<PC1);	//init PC1 to be input

    DDRC &= ~(1<<PC2);	//init PC2 to be input

    DDRC &= ~(1<<PC4);	//init PC4 to be input

    DDRC &= ~(1<<PC5);	//init PC5 to be input

    PORTC |=(1<<PC0)|(1<<PC1)|(1<<PC2)|(1<<PC4)|(1<<PC5);//open pull-up register's PC0,PC1,PC2,PC4,PC5

    DDRB &= ~(1<<PB0);	//init PC0 to be input

    DDRB &= ~(1<<PB1);	//init PC1 to be input

    DDRB &= ~(1<<PB2);	//init PC2 to be input

    DDRB &= ~(1<<PB4);	//init PC4 to be input

    DDRB &= ~(1<<PB5);	//init PC5 to be input

    PORTB |=(1<<PB0)|(1<<PB1)|(1<<PB2)|(1<<PB4)|(1<<PB5);//open pull-up register's PB0,PB1,PB2,PB4,PB5

    usbInit();

    /* enforce re-enumeration, do this while interrupts are disabled! */
    usbDeviceDisconnect();
    _delay_ms(300);
    usbDeviceConnect();

    /* enable global interrupts */
    sei();

    /* main event loop */
    for(;;)
    {
        /* always have report ready */
        buildReport();

        usbPoll();

        /* let the host know without asking */
        if (usbInterruptIsReady())
        {
            usbSetInterrupt((void *)&reportBuffer, sizeof(reportBuffer));
        }
    }

    return 0;
}

/* ------------------------------------------------------------------------- */
